import { Injectable } from '@angular/core';
import { ActivatedRouteSnapshot, CanActivate, Router, RouterStateSnapshot } from '@angular/router';
import { catchError } from 'rxjs/operators';
import { AuthService } from 'src/app/services/auth/auth.service';
import { ThemeService } from 'src/app/services/theme/theme.service';

@Injectable({
  providedIn: 'root'
})
export class AuthGuard implements CanActivate {
  constructor(
    private router: Router,
    private authService: AuthService,
    private themeService: ThemeService
  ) {
  }

  async canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): Promise<boolean> {
    if (!this.authService.getToken().token) {
      this.authService.logout();
      return false;
    }

    if (!await this.authService.isUserLoggedInAsync()) {
      this.authService.logout();
      return false;
    }

    const role = route.data['role'];
    if (role) {
      this.authService.hasUserPermission(role).then(hasPermission => {
        if (!hasPermission) {
          this.router.navigate(['/dashboard']);
          return false;
        }
        return true;
      });
    }
    return true;
  }
}
